# This program asks for the board serial number, along with it's bus address,
# it then creates a calibration source file, assembles it, puts it in the eeprom
# along with an oscillator calibration program, It then runs the calibration 
# routine, which modifies the eeprom with the new value. It then checks to make
# sure that the eeprom has been modified, and if so, it programs in the final
# program.

# Make sure your programs are located here, or change these lines appropriately!
$avrasm2 = qq("c:\\Program Files\\Atmel\\AVR Tools\\AvrAssembler2\\avrasm2");
$stk500 = qq("c:\\Program Files\\Atmel\\AVR Tools\\STK500\\stk500.exe");
$COMPORT = "USB";
$finalcode = "..\\software\\BRMK3.hex";

printf "BrassRatMK3 Initialize Program\n\n";
printf "Serial Number? ";
$serialnum = <>;
chomp $serialnum;

printf "Bus Address ";
$busaddress = <>;
chomp $busaddress;
$asmfile = sprintf "P%04d.asm", $serialnum;
$eepfile = sprintf "P%04d.eep", $serialnum;
$hexfile = sprintf "P%04d.eep", $serialnum;
open (OUTPUTFILE, ">" . $asmfile);

printf OUTPUTFILE ";*****************************************************************\n";
printf OUTPUTFILE ";\n";
printf OUTPUTFILE "; Calibration data for unit %s\n", $asmfile;
printf OUTPUTFILE ";\n";
printf OUTPUTFILE ";*****************************************************************\n\n";
printf OUTPUTFILE "                 .device ATMEGA168\n";
printf OUTPUTFILE "                 .Eseg\n";
printf OUTPUTFILE "                 .org    0\n";
printf OUTPUTFILE "V_off:           .dw     0\n";
printf OUTPUTFILE "V_cal:           .dw     2504\n";
printf OUTPUTFILE "T_off:           .dw     0\n";
printf OUTPUTFILE "T_cal:           .dw     500\n";
printf OUTPUTFILE "V2_off:          .dw     0\n";
printf OUTPUTFILE "V2_cal:          .dw     2504\n";
printf OUTPUTFILE "V3_off:          .dw     0\n";
printf OUTPUTFILE "V3_cal:          .dw     2504\n";
printf OUTPUTFILE "Low_volts:       .dw     409\n";
printf OUTPUTFILE "High_volts:      .dw     605\n";
printf OUTPUTFILE "hot_temp:        .dw     307\n";
printf OUTPUTFILE "S_num:           .dw     %d\n", $serialnum;
printf OUTPUTFILE "                 .dw     %d\n", int($serialnum / 65536);
printf OUTPUTFILE "B_add:           .db     %d\n", $busaddress;
printf OUTPUTFILE "OSCCAL:          .db     0xff\n";
close OUTPUTFILE;

my $returncode = 0;
$asm = qq($avrasm2 -e $eepfile $asmfile);
$returncode = (system $asm) >> 8;
if ($returncode != 0) {
	die "Cal file assembly failed :$returncode";
}

# allow erasure of eeprom
$prog = qq($stk500 -c$COMPORT -I125000 -dATMEGA168 -e -fdcc2 -EFD);
$returncode = (system $prog) >> 8;
if ($returncode != 0) {
	die "Allowing eeprom erase failed :$returncode";
}

# Pour in the Oscillator calibration program...
$prog = qq($stk500 -c$COMPORT -I1000000 -dATMEGA168 -ifRC_Calibration.hex -ie$hexfile -e -fd4c2 -EFD -pf -pe);
$returncode = (system $prog) >> 8;
if ($returncode != 0) {
	die "Initial programming failed :$returncode";
}

# And calibrate the oscillator
$prog = qq($stk500 -c$COMPORT -dATMEGA168 -Y);
$returncode = (system $prog) >> 8;
if ($returncode != 0) {
	die "Oscillator calibration failed :$returncode";
}

# make sure we did change the osccal byte
printf " vv IGNORE errors below vv\n\n";
$prog = qq($stk500 -c$COMPORT -I1000000 -dATMEGA168 -ae01b,01b -ve -ie$hexfile);
$returncode = (system $prog) >> 8;
if ($returncode == 0) {
	die "EEProm wasn't modified! :$returncode";
}
printf " ^^ IGNORE errors above ^^\n\n";

# Now put in the real bits...
$prog = qq($stk500 -c$COMPORT -I1000000 -dATMEGA168 -if$finalcode -e -pf);
$returncode = (system $prog) >> 8;
if ($returncode != 0) {
	die "Final programming failed :$returncode";
}

printf "\n\n Final Programming SUCCEEDED! YIPPEE!\n";

